--[[
	Events.lua
		This handles global events and the loading of Bagnon and Banknon
--]]

function BagnonEvents_OnLoad()
	--Slash handler
	BagnonSlash_Load();
	
	--Rescaling detection, for whenever the UI scale changes
	Infield.AddRescaleAction(BagnonEvents_RescaleAll);
	
	--Events
	this:RegisterEvent("ADDON_LOADED");
end

--[[
	Event Handler
--]]

function BagnonEvents_OnEvent(event)

	if ( event == "BAG_UPDATE" or event == "ITEM_LOCK_CHANGED" or event == "BAG_UPDATE_COOLDOWN") then
	
		if( Bagnon and Bagnon:IsVisible() ) then
			BagnonCore_Update(Bagnon);		
		end
		
		if( Banknon and Banknon:IsVisible() ) then
			BagnonCore_Update(Banknon);
		end
		
	elseif( event == "BANKFRAME_OPENED") then
	
		if( IsAddOnEnabled("Bagnon") and not IsAddOnLoaded("Bagnon") ) then
			LoadAddOn("Bagnon");
		end
		
		if( IsAddOnEnabled("Banknon") and BagnonSets.overrideBank ) then
			if(not Banknon) then
				LoadAddOn("Banknon");
			end
		else
			BagnonEvents_ShowBlizBank();
		end
		
	--the following are tweaks because I read that bag update and item lock changed fire off lots of times when zoning
	elseif(event == "PLAYER_ENTERING_WORLD") then
		
		this:RegisterEvent("BAG_UPDATE");
		this:RegisterEvent("ITEM_LOCK_CHANGED");
		this:RegisterEvent("BAG_UPDATE_COOLDOWN");
		
		if(Bagnon and Bagnon:IsVisible() ) then
			BagnonCore_Update(Bagnon);
		end

		if(Banknon and Banknon:IsVisible() ) then
			BagnonCore_Update(Banknon)
		end	
		
	elseif(event == "PLAYER_LEAVING_WORLD") then
	
		this:UnregisterEvent("BAG_UPDATE");
		this:UnregisterEvent("ITEM_LOCK_CHANGED");
		this:UnregisterEvent("BAG_UPDATE_COOLDOWN");
		
	elseif( event == "ADDON_LOADED" and arg1 == "Bagnon_Core") then
	
		--disable the ItemOnUpdate function for those using KCItems
		--currently, it causes a pretty bad memory leak
		if(KC_Items) then
			BagnonItem_OnUpdate = function() return; end;
		end

		this:UnregisterEvent("ADDON_LOADED");
		
		this:RegisterEvent("PLAYER_LEAVING_WORLD");
		this:RegisterEvent("PLAYER_ENTERING_WORLD");
		
		--dynamically loading frame stuff
		this:RegisterEvent("BANKFRAME_OPENED");
		
		BagnonEvents_Load();
	end
end

function BagnonEvents_Load()
	BagnonEvents_LoadVariables();
	
	--unregister the bank frame
	BankFrame:UnregisterEvent("BANKFRAME_OPENED");
	
	--[[ Workaround to get some localized names ]]--
	if( not (GetLocale() == "enUS" or GetLocale() == "deDE"or BagnonSets.noDebug) ) then
		BagnonMsg("Obtaining localized data.  Please report the following to where you downloaded Bagnon from.");
		BagnonMsg( GetLocale() );
	end
	
	--backpack
	local name, _, _, _, iType, subType = GetItemInfo(4500); 		
	if(name) then
		if( not (GetLocale() == "enUS" or GetLocale() == "deDE"or BagnonSets.noDebug) ) then
			BagnonMsg( "Backpack:  " .. (iType or "null") .. ", " .. (subType or "null") );
		end
		if(iType) then
			BAGNON_ITEMTYPE_CONTAINER = iType;
			BAGNON_SUBTYPE_BAG = subType;
		end
	end

	--ammo	
	name, _, _, _, iType, subType = GetItemInfo(8218); 
	if(name) then
		if( not (GetLocale() == "enUS" or GetLocale() == "deDE"or BagnonSets.noDebug) ) then
			BagnonMsg( "Ammo:  " .. (iType or "null") .. ", " .. (subType or "null") );
		end
		if(iType) then
			BAGNON_ITEMTYPE_QUIVER = iType;
		end
	end

	--soul pouch
	name, _, _, _, iType, subType = GetItemInfo(21340); 
	if(name) then
		if( not (GetLocale() == "enUS" or GetLocale() == "deDE"or BagnonSets.noDebug) ) then
			BagnonMsg( "Soul Bag:  " .. (iType or "null") .. ", " .. (subType or "null") );
		end
		if(subType) then
			BAGNON_SUBTYPE_SOULBAG = iType;
		end
	end
end

function BagnonEvents_LoadVariables()
	if(not BagnonSets) then	
		BagnonSets = {
			overrideBank = 1,
			showTooltips = 1,
			version = BAGNON_VERSION,
		};
		BagnonMsg(BAGNON_INITIALIZED);
	elseif(BagnonSets.version ~= BAGNON_VERSION) then
		BagnonSets.version = BAGNON_VERSION;
		BagnonMsg(BAGNON_UPDATED);
	end
end

function BagnonEvents_RescaleAll()
	if(Bagnon) then
		BagnonCore_Reposition(Bagnon);
	end
	
	if(Banknon) then
		BagnonCore_Reposition(Banknon);
	end
end

--[[
	Taken from Blizzard's code
	Shows the normal bank frame
--]]

function BagnonEvents_ShowBlizBank()
	BankFrameTitleText:SetText(UnitName("npc"));
	SetPortraitTexture(BankPortraitTexture,"npc");
	ShowUIPanel(BankFrame);
	
	if( not BankFrame:IsVisible() ) then
		CloseBankFrame();
	end
	UpdateBagSlotStatus();
end