--[[

	QuickLoot: easier, faster looting
		copyright 2004 by Telo

	- Automatically positions the most relevant part of the loot window under your cursor
	
]]

--------------------------------------------------------------------------------------------------
-- Configuration variables
--------------------------------------------------------------------------------------------------

QuickLoot_AutoHide = 1;	-- set to 1 if you want the window to auto-hide when there's no loot or
						-- set to nil if you don't want this behavior
-- Ascendant 08.05.05 - added the following varaibles
QL_VarLoaded = false;	-- have variables been loaded?
QL_ModLoaded = false;	-- has the mod been loaded?
QL_ThisChar = nil;		-- stores name of char

--------------------------------------------------------------------------------------------------
-- Local variables
--------------------------------------------------------------------------------------------------

-- Function hooks
local originalLootFrame_OnEvent;
local originalLootFrame_Update;

--------------------------------------------------------------------------------------------------
-- Internal functions
--------------------------------------------------------------------------------------------------
local function QuickLoot_ItemUnderCursor()
	local index;
	local x, y = GetCursorPosition();
	local scale = LootFrame:GetEffectiveScale();

	x = x / scale;
	y = y / scale;

	LootFrame:ClearAllPoints();

	for index = 1, LOOTFRAME_NUMBUTTONS, 1 do
		local button = getglobal("LootButton"..index);
		if( button:IsVisible() ) then
			x = x - 42;
			y = y + 56 + (40 * index);
			LootFrame:SetPoint("TOPLEFT", "UIParent", "BOTTOMLEFT", x, y);
			return;
		end
	end

	if( LootFrameDownButton:IsVisible() ) then
		-- If down arrow, position on it
		x = x - 158;
		y = y + 223;
	else
		if( QuickLoot_Config[QL_ThisChar].AutoHide and GetNumLootItems() == 0 ) then	-- Ascendant 08.05.05 - changed variable
			HideUIPanel(LootFrame);
			return
		end
		-- Otherwise, position on close box
		x = x - 173;
		y = y + 25;
	end
	
	LootFrame:SetPoint("TOPLEFT", "UIParent", "BOTTOMLEFT", x, y);
end

local function QuickLoot_LootFrame_OnEvent(event)
	originalLootFrame_OnEvent(event);
	
	if ( event == "LOOT_SLOT_CLEARED" ) then
		-- Ascendant 08.05.05 - if the mod is disabled ingame then the WoW default functionality of the loot frame should work
		if (QuickLoot_Config[QL_ThisChar].Enable) then
			if (not QuickLoot_Config[QL_ThisChar].DontMove) then	-- Ascendant 08.05.05 - changed variable
				QuickLoot_ItemUnderCursor();
			end
		end
	end
	--[[if (event == "LOOT_OPENED") then
		-- Ascendant 08.05.05 - if the mod is disabled ingame then the WoW default functionality of the loot frame should work
		if (QuickLoot_Config[QL_ThisChar].Enable) then
			if (GetNumLootItems() == 0) then
				if (QuickLoot_Config[QL_ThisChar].AutoHide) then	-- Ascendant 08.05.05 - changed variable
					HideUIPanel(LootFrame);
				end
			elseif (QuickLoot_Config[QL_ThisChar].DontMove) then	-- Ascendant 08.05.05 - changed variable
				QuickLoot_ItemUnderCursor();
			end
		end
	end
	originalLootFrame_OnEvent(event);
	if ( event == "LOOT_SLOT_CLEARED" ) then
		QuickLoot_ItemUnderCursor();
	end]]
end

local function QuickLoot_LootFrame_Update()
	originalLootFrame_Update();
	if (QuickLoot_Config[QL_ThisChar].Enable) then
		if (GetNumLootItems() == 0) then
			if (QuickLoot_Config[QL_ThisChar].AutoHide) then	-- Ascendant 08.05.05 - changed variable
				HideUIPanel(LootFrame);
			end
		elseif (not QuickLoot_Config[QL_ThisChar].DontMove) then	-- Ascendant 08.05.05 - changed variable
			QuickLoot_ItemUnderCursor();
		end
	end
	--[[originalLootFrame_Update();
	QuickLoot_ItemUnderCursor();]]
end

--------------------------------------------------------------------------------------------------
-- OnFoo functions
--------------------------------------------------------------------------------------------------
function QuickLoot_OnLoad()
	-- Hook the LootFrame_OnEvent function
	originalLootFrame_OnEvent = LootFrame_OnEvent;
	LootFrame_OnEvent = QuickLoot_LootFrame_OnEvent;
	
	-- Hook the LootFrame_Update function
	originalLootFrame_Update = LootFrame_Update;
	LootFrame_Update = QuickLoot_LootFrame_Update;
	
	-- Ascendant 08.05.05 - added event registration
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("UNIT_NAME_UPDATE");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	-- Don't treat the LootFrame as a UI panel
	UIPanelWindows["LootFrame"] = nil;
	
	-- Ascendant 08.05.05 - added slash command handler
	SLASH_QUICKLOOT1 = "/quickloot";
	SlashCmdList["QUICKLOOT"] = function(msg)
		QuickLoot_SlashCmdHndlr(msg);
	end
	if( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("Telo's QuickLoot AddOn loaded (modified by Ascendant)");
	end
	UIErrorsFrame:AddMessage("Telo's QuickLoot AddOn loaded (modified by Ascendant)", 1.0, 1.0, 1.0, 1.0, UIERRORS_HOLD_TIME);
end
